from django.contrib.gis.db import models
import oxlib
import flickr
import settings
try:
  import json
except:
  import simplejson as json
from oxweb import youtube

flickr.API_KEY = settings.FLICKR_API_KEY
flickr.SECRET = settings.FLICKR_SECRET

class Flyover(models.Model):
  point = models.PointField()
  name = models.CharField(max_length=255)
  txt = models.TextField(blank=True, null=True)
  objects = models.GeoManager()

  def __unicode__(self):
    return self.name

  def geojson_as_dict(self):
    return {
      'type': 'Feature',
      'geometry': {'type': 'Point', 'coordinates': [self.point.x, self.point.y]},
      'properties': {'id': self.id}
      }

  def get_media(self):
    d = {
      'images': []
    }
    images = Image.objects.filter(flyover=self)
    for i in images:
      d['images'].append(i.get_dict())
    return d

class Image(models.Model):
  url = models.URLField("FlickR URL")
  caption = models.CharField(max_length=255, blank=True, null=True)
  category = models.ForeignKey("ImageCategory")
  flyover = models.ForeignKey(Flyover)
  flickrData = models.TextField(blank=True)
  
  def get_flickr_id(self):
    regex = r'http\:\/\/.*?\/\d{4}/(.*?)_.*'
    return oxlib.findRe(self.url, regex)

  def __unicode__(self):
    return self.url + " - " + self.caption + " - " + self.category.name

  def get_dict(self):
    if self.flickrData != '':
      return json.loads(self.flickrData)
    else:
      flickrPhoto = flickr.Photo(id=self.get_flickr_id())
      sizes = flickrPhoto.getSizes()
      ret = {
      'url': self.url,
      'sizes': sizes,
      'caption': self.caption,
      'flyover': {
        'id': self.flyover.id,
        'name': self.flyover.name
        }
      }
      self.flickrData = json.dumps(ret)
      self.save()
      return ret
      

class ImageCategory(models.Model):
  name = models.CharField(max_length=255)
  description = models.TextField(blank=True, null=True)

  def __unicode__(self):
    return self.name

  def get_dict(self):
    images = []
    flyovers = []
    for i in Image.objects.filter(category=self):
      images.append(i.get_dict())
      flyovers.append(i.flyover.geojson_as_dict())
    return {
      'images': images,
      'flyovers': {
        'type': 'FeatureCollection',
        'features': flyovers
      }
    }

class Video(models.Model):
  url = models.URLField("Youtube URL")
  caption = models.CharField(max_length=255, blank=True, null=True)
  category = models.ForeignKey("VideoCategory")
  flyover = models.ForeignKey(Flyover)
  youtubeData = models.TextField(blank=True)

  def __unicode__(self):
    return self.url + " - " + self.caption + " - " + self.category.name

  def get_youtube_id(self):
    reg = r'http.*?v=(.*?)$'
    return oxlib.findRe(self.url, reg)

  def get_dict(self):
    if self.youtubeData != '':
      return json.loads(self.youtubeData)
    else:
      youtube_id = self.get_youtube_id()
      info = youtube.getMovieInfo(youtube_id)
      info['title'] = '' #Nasty, to avoid UnicodeDecodeError's when trying to do json.dumps on some video titles. FIXME.
      ret = {
      'info': info,
      'flyover': {
        'id': self.flyover.id,
        'name': self.flyover.name
        }
      }
      self.youtubeData = json.dumps(ret)
      self.save()
      return ret


class VideoCategory(models.Model):
  name = models.CharField(max_length=255)
  description = models.TextField(blank=True, null=True)

  def __unicode__(self):
    return self.name

  def get_dict(self):
    videos = []
    flyovers = []
    for v in Video.objects.filter(category=self):
      videos.append(v.get_dict())
      flyovers.append(v.flyover.geojson_as_dict())
    return {
      'videos': videos,
      'flyovers': {
        'type': 'FeatureCollection',
        'features': flyovers
      }
    }
    

class Audio(models.Model):
  fil = models.FileField(upload_to='audio/')
  caption = models.CharField(max_length=255, blank=True, null=True)
  category = models.ForeignKey("AudioCategory")
  flyover = models.ForeignKey(Flyover)

  def __unicode__(self):
    return self.url + " - " + self.caption + " - " + self.category.name

class AudioCategory(models.Model):
  name = models.CharField(max_length=255)
  description = models.TextField(blank=True, null=True)

  def __unicode__(self):
    return self.name

class Text(models.Model):
#  url = models.URLField("FlickR URL")
  text = models.TextField()
  category = models.ForeignKey("TextCategory")
  flyover = models.ForeignKey(Flyover)

  def __unicode__(self):
    return self.text + " - " + self.category.name

class TextCategory(models.Model):
  name = models.CharField(max_length=255)
  description = models.TextField(blank=True, null=True)

  def __unicode__(self):
    return self.name

# Create your models here.
